@@ -2,7 +2,7 @@
 """
 A test harness for gnupg.py.
 
-Copyright (C) 2008-2012 Vinay Sajip. All rights reserved.
+Copyright (C) 2008-2013 Vinay Sajip. All rights reserved.
 """
 import doctest
 import logging
@@ -16,12 +16,14 @@ import unittest
 import gnupg
 
 __author__ = "Vinay Sajip"
-__date__  = "$01-Mar-2010 12:30:04$"
+__date__  = "$11-Mar-2013 23:48:26$"
 
 ALL_TESTS = True
 
 logger = logging.getLogger(__name__)
 
+GPGBINARY = os.environ.get('GPGBINARY', 'gpg')
+
 KEYS_TO_IMPORT = """-----BEGIN PGP PUBLIC KEY BLOCK-----
 Version: GnuPG v1.4.9 (MingW32)
 
@@ -96,7 +98,9 @@ class GPGTestCase(unittest.TestCase):
                             "Not a directory: %s" % hd)
             shutil.rmtree(hd)
         self.homedir = hd
-        self.gpg = gnupg.GPG(gnupghome=hd, gpgbinary='gpg')
+        self.gpg = gnupg.GPG(gnupghome=hd, gpgbinary=GPGBINARY)
+        if self.gpg.version and self.gpg.version >= (2,):
+            self.gpg.options = ['--debug-quick-random']
 
     def test_environment(self):
         "Test the environment by ensuring that setup worked"
@@ -125,6 +129,10 @@ class GPGTestCase(unittest.TestCase):
             'Name-Comment': 'A test user',
             'Expire-Date': 0,
         }
+        if '--debug-quick-random' in (self.gpg.options or []):
+            # If using the fake RNG, a key isn't regarded as valid
+            # unless its comment has the text (insecure!) in it.
+            params['Name-Comment'] = 'A test user (insecure!)'
         params['Name-Real'] = '%s %s' % (first_name, last_name)
         params['Name-Email'] = ("%s.%s@%s" % (first_name, last_name, domain)).lower()
         if passphrase is None:
@@ -138,7 +146,71 @@ class GPGTestCase(unittest.TestCase):
         result = self.generate_key("Barbara", "Brown", "beta.com")
         self.assertNotEqual(None, result, "Non-null result")
         return result
-    
+
+    def test_key_generation_with_invalid_key_type(self):
+        "Test that key generation handles invalid key type"
+        params = {
+            'Key-Type': 'INVALID',
+            'Key-Length': 1024,
+            'Subkey-Type': 'ELG-E',
+            'Subkey-Length': 2048,
+            'Name-Comment': 'A test user',
+            'Expire-Date': 0,
+            'Name-Real': 'Test Name',
+            'Name-Email': 'test.name@example.com',
+        }
+        cmd = self.gpg.gen_key_input(**params)
+        result = self.gpg.gen_key(cmd)
+        self.assertFalse(result.data, 'Null data result')
+        self.assertEqual(None, result.fingerprint, 'Null fingerprint result')
+
+    def test_key_generation_with_colons(self):
+        "Test that key generation handles colons in key fields"
+        params = {
+            'key_type': 'RSA',
+            'name_real': 'urn:uuid:731c22c4-830f-422f-80dc-14a9fdae8c19',
+            'name_comment': 'dummy comment',
+            'name_email': 'test.name@example.com',
+        }
+        cmd = self.gpg.gen_key_input(**params)
+        result = self.gpg.gen_key(cmd)
+        keys = self.gpg.list_keys()
+        self.assertEqual(len(keys), 1)
+        key = keys[0]
+        uids = key['uids']
+        self.assertEqual(len(uids), 1)
+        uid = uids[0]
+        self.assertEqual(uid, 'urn:uuid:731c22c4-830f-422f-80dc-14a9fdae8c19 '
+                              '(dummy comment) <test.name@example.com>')
+
+    def test_key_generation_with_escapes(self):
+        cmd = self.gpg.gen_key_input(name_comment='Funny chars: '
+                                                  '\\r\\n\\f\\v\\0\\b',
+                                     name_real='Test Name',
+                                     name_email='test.name@example.com')
+        result = self.gpg.gen_key(cmd)
+        keys = self.gpg.list_keys()
+        self.assertEqual(len(keys), 1)
+        key = keys[0]
+        uids = key['uids']
+        self.assertEqual(len(uids), 1)
+        uid = uids[0]
+        self.assertEqual(uid, 'Test Name (Funny chars: '
+                              '\r\n\x0c\x0b\x00\x08) <test.name@example.com>')
+
+    def test_key_generation_with_empty_value(self):
+        "Test that key generation handles empty values"
+        params = {
+            'key_type': 'RSA',
+            'key_length': 1024,
+            'name_comment': ' ', # Not added, so default will appear
+        }
+        cmd = self.gpg.gen_key_input(**params)
+        self.assertTrue('\nName-Comment: Generated by gnupg.py\n' in cmd)
+        params['name_comment'] = 'A'
+        cmd = self.gpg.gen_key_input(**params)
+        self.assertTrue('\nName-Comment: A\n' in cmd)
+        
     def test_list_keys_after_generation(self):
         "Test that after key generation, the generated key is available"
         self.test_list_keys_initial()
@@ -159,6 +231,7 @@ class GPGTestCase(unittest.TestCase):
         key = self.generate_key("Barbara", "Brown", "beta.com")
         barbara = key.fingerprint
         gpg = self.gpg
+        gpg.encoding = 'latin-1'
         if gnupg._py3k:
             data = 'Hello, André!'
         else:
@@ -189,7 +262,8 @@ class GPGTestCase(unittest.TestCase):
         logger.debug("test_import_and_export begins")
         self.test_list_keys_initial()
         gpg = self.gpg
-        gpg.import_keys(KEYS_TO_IMPORT)
+        result = gpg.import_keys(KEYS_TO_IMPORT)
+        self.assertEqual(result.summary(), '2 imported')
         public_keys = gpg.list_keys()
         self.assertTrue(is_list_with_len(public_keys, 2),
                         "2-element list expected")
@@ -238,6 +312,7 @@ class GPGTestCase(unittest.TestCase):
         "Test that signing and verification works"
         logger.debug("test_signature_verification begins")
         key = self.generate_key("Andrew", "Able", "alpha.com")
+        self.gpg.encoding = 'latin-1'
         if gnupg._py3k:
             data = 'Hello, André!'
         else:
@@ -253,6 +328,8 @@ class GPGTestCase(unittest.TestCase):
             logger.debug("ver: %r", verified.fingerprint)
         self.assertEqual(key.fingerprint, verified.fingerprint,
                          "Fingerprints must match")
+        self.assertEqual(verified.trust_level, verified.TRUST_ULTIMATE)
+        self.assertEqual(verified.trust_text, 'TRUST_ULTIMATE')
         if not os.path.exists('random_binary_data'):
             data_file = open('random_binary_data', 'wb')
             data_file.write(os.urandom(5120 * 1024))
@@ -309,6 +386,13 @@ class GPGTestCase(unittest.TestCase):
         self.assertRaises(ValueError, gnupg.GPG, gnupghome=self.homedir,
                           gpgbinary='frob')
 
+    def test_make_args(self):
+        "Test argument line construction"
+        self.gpg.options = ['--foo', '--bar']
+        args = self.gpg.make_args(['a', 'b'], False)
+        self.assertTrue(len(args) > 4)
+        self.assertEqual(args[-4:], ['--foo', '--bar', 'a', 'b'])
+
     def test_file_encryption_and_decryption(self):
         "Test that encryption/decryption to/from file works"
         logger.debug("test_file_encryption_and_decryption begins")
@@ -356,10 +440,14 @@ TEST_GROUPS = {
     'crypt' : set(['test_encryption_and_decryption',
                    'test_file_encryption_and_decryption']),
     'key' : set(['test_deletion', 'test_import_and_export',
-                 'test_list_keys_after_generation']),
+                 'test_list_keys_after_generation',
+                 'test_key_generation_with_invalid_key_type',
+                 'test_key_generation_with_escapes',
+                 'test_key_generation_with_empty_value',
+                 'test_key_generation_with_colons']),
     'import' : set(['test_import_only']),
     'basic' : set(['test_environment', 'test_list_keys_initial',
-                   'test_nogpg']),
+                   'test_nogpg', 'test_make_args']),
 }
 
 def suite(args=None):
@@ -390,8 +478,9 @@ def init_logging():
 def main():
     init_logging()
     tests = suite()
-    unittest.TextTestRunner(verbosity=2).run(tests)
+    results = unittest.TextTestRunner(verbosity=2).run(tests)
+    return not results.wasSuccessful()
 
 
 if __name__ == "__main__":
-    main()
+    sys.exit(main())
